function density, r0, h, alphain, alphaout, x, y, z

  n0 = 1.
  r = sqrt(x*x + y*y + z*z)
  ror0 = r / r0

  n = (n0 * sqrt(2.)) * exp((-abs(z)/(h * r))^2.) / sqrt((ror0)^(-2.*alphain) + (ror0)^(-2.*alphaout))

  return, n
  
end

pro dustmap_point_maker

  ;--------------------
  ;      INPUTS
  ;--------------------
  output_file = 'test.txt'
  inclination = 0. ;degrees
  numpixels = 400
  pixelsize = 1. ;AU
  znumpixels = 100
  zpixelsize = 1. ;AU
  oversample = 1                ;must be an integer

  r0 = 40.                      ;AU
  h = 0.1
  alphain = 10.
  alphaout = -4.
  tol = 1e-4 
  ;--------------------



  ;Make a few calculations up front  
  cosinc = cos(inclination * !dpi/180.)
  sininc = sin(inclination * !dpi/180.)
  npix = long(numpixels) * oversample
  pixsize = float(pixelsize) / oversample
  znpix = long(znumpixels)
  zpixsize = float(zpixelsize)
  
  ;Generate points in the image frame (primed frame)
  xp = (findgen(npix)+0.5) - npix/2.
  xp = [[xp],[xp]]
  xp = rebin(xp,npix,npix)
  yp = rotate(xp,1)
  xp = [[[xp]],[[xp]]]
  xp = rebin(xp,npix,npix,znpix)
  yp = [[[yp]],[[yp]]]
  yp = rebin(yp,npix,npix,znpix)
  zp = yp*0

  zp_list = (findgen(znpix)+0.5) - znpix/2.
  for i=0,znpix-1 do zp[*,*,i] = zp_list[i]
  
  
  ;Generate z points for each of those x,y combinations
  ;To avoid huge amounts of memory, we keep this separate

  ;Make some dummy particle IDs
  particle_IDs = intarr(long(npix)*long(npix)*long(znpix))
  
  append = 0
  silent = 0

  ;rotate about x-axis by inclination
  x = xp
  y = yp * cosinc + zp * sininc
  z = -yp * sininc + zp * cosinc

  ;Calculate density
  intensity = density(r0, h, alphain, alphaout, x, y, z)
  maxintensity = max(intensity)
  
  ;Keep points that contribute
  j = where(intensity gt tol*maxintensity)  
  intensity /= max(intensity[j])
  dustmap_binary_writer, output_file, particle_IDs[j], x[j], y[j], z[j], intensity=intensity[j]  

end
