void hgphasefunction(float *g, float *w, int numg, float *costheta, int numthetas, int nlambda, float *pfunc){

  // hgphasefunction.c
  // Calculates the Henyey-Greenstein phase function
  //
  // Input:
  // g: an array of the HG phase function asymmetry parameters
  // w: an array of the weights assigned to each g value
  // numg: the number of g values
  // costheta: an array of costheta values, the cosine
  //           of the scattering angle
  // numthetas: the number of entries in costheta array
  //
  // Output:
  // pfunc: an array of scattering phase function amplitudes,
  //        with a 1:1 correspoding to the costheta values
  //
  // Created by Christopher Stark
  // NASA GSFC
  // christopher.c.stark@nasa.gov
  // Last updated 18 Nov 2014 by Christopher Stark

  int i, ig;
  float tempval;
 
  //calculate phase function as a function of costheta
  for(i=0;i<numthetas;i++){
    for(ig=0;ig<numg;ig++){
      tempval = (1.0 - 2.0 * g[ig] * costheta[i] + g[ig] * g[ig]);
      tempval = w[ig] * (1.0 - g[ig] * g[ig]) / sqrt(tempval*tempval*tempval) / (4 * PI);
      if(ig==0) {
	pfunc[i*nlambda] = tempval;
      } else {
	pfunc[i*nlambda] += tempval;
      }
    }
  }

}
